/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.awt.Insets;
import java.io.File;

import com.topcoder.util.config.*;
import com.topcoder.util.image.svg2dgraphics.SVGGraphics2D;

import junit.framework.TestCase;

/**
 * Unit test cases for FixedGridLayoutManager.
 *
 * Testing of FixedGridLayoutManager includes the accuracy testing of the layout algorithm,
 * to verify that each parameter works independently.
 *
 * @author TCSDeveloper
 */
public class FixedGridLayoutManagerUnitTests extends TestCase {

    /**
     * The filename to persist to.
     */
    private String filename = "test_files/unittests/unittest-";

    /**
     * FixedGridLayoutManager instance to test.
     */
    private FixedGridLayoutManager manager = null;

    /**
     * ShapeFactory instance to use.
     */
    private ShapeFactory factory = null;

    /**
     * RendererGraphics to use.
     */
    private RendererGraphics graphics = null;

    /**
     * Create new layout manager, shape factory and renderer graphics for each test.
     */
    protected void setUp() {
        manager = new FixedGridLayoutManager();
        factory = new ShapeFactory();        
    }

    /**
     * Remove the file and dispose the renderer graphics.
     */
    protected void tearDown() {
        if (graphics != null) 
    	{
        	graphics.dispose();
    	}
    }

    /**
     * Create instance.
     */
    public void testCreate() {    	
        manager = new FixedGridLayoutManager();
    }

    /**
     * Create instance with colPadding and rowPadding.
     */
    public void testCreate_ColPaddingRowPadding() {
        manager = new FixedGridLayoutManager();
    }

    /**
     * Create instance with colPadding, rowPadding and textMargin.
     */
    public void testCreate_ColPaddingRowPaddingTextMargin() {
        manager = new FixedGridLayoutManager(10, 10, new Insets(10, 10, 10, 10));
    }

    /**
     * Create instance with negative colPadding.
     */
    public void testCreate_NegativeColPadding2() {
        try {
            new FixedGridLayoutManager(-1, 10, new Insets(10, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative rowPadding.
     */
    public void testCreate_NegativeRowPadding2() {
        try {
            new FixedGridLayoutManager(10, -1, new Insets(10, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with null textMargin.
     */
    public void testCreate_NullTextMargin() {
        try {
            new FixedGridLayoutManager(10, 10, null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Create instance with negative top.
     */
    public void testCreate_NegativeTop() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(-1, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative left.
     */
    public void testCreate_NegativeLeft() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, -1, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative bottom.
     */
    public void testCreate_NegativeBottom() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, 10, -1, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative right.
     */
    public void testCreate_NegativeRight() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, 10, 10, -1));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Layout single dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SingleDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        
        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-single_dependency"));       
        
        manager.layout(dependency, factory, render);
        graphics.complete();
    }

    /**
     * Layout dependency tree.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_DependencyTree() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));
        
        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-dependency_tree"));        
        
        manager.layout(dependency, factory, render);
        graphics.complete();
    }

    /**
     * Layout seen dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SeenDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(dependency);

        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-seen_dependency"));        

        manager.layout(dependency, factory, render);
        graphics.complete();
    }

    /**
     * Layout shapeOverride.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ShapeOverride() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer", ShapeFactory.SEEN_ELEMENT);
        
        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-shape_override"));        

        manager.layout(dependency, factory, render);  
        
        graphics.complete();
    }

    /**
     * Layout colPadding and rowPadding.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ColPaddingRowPadding() throws RendererException {    	
        manager = new FixedGridLayoutManager();
        manager.setColPadding(0);
        manager.setRowPadding(50);
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-colpadding_rowpadding"));        

        manager.layout(dependency, factory, render);  
        graphics.complete();
    }

    /**
     * Layout textMargin.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_TextMargin() throws RendererException {
        manager = new FixedGridLayoutManager(10, 10, new Insets(50, 0, 50, 0));
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-text_margin"));        

        manager.layout(dependency, factory, render);       

        graphics.complete();
    }

    /**
     * Test layout() with null dependency.
     */
    public void testLayout_NullDependency() {
        try {
        	SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-text_margin"));       
            manager.layout(null, factory, render);
      
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        } catch (Exception e) {
        	fail("Should have thrown NullPointerException.");
        }
    }

    /**
     * Test layout() with null shape factory.
     */
    public void testLayout_NullShapeFactory() {
        try {
        	SVGRenderer render = new SVGRenderer(new File("test_files/unittests/unittest-text_margin"));
            manager.layout(new Dependency("a"), null, render);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        } catch (Exception e) {
        	fail("Should have thrown NullPointerException.");
        }
    }

    /**
     * Test layout() with null renderer graphics.
     */
    public void testLayout_NullRendererGraphics() {
        try {
            manager.layout(new Dependency("a"), factory, null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        } catch (Exception e) {
        	fail("Should have thrown NullPointerException.");
        }
    }

}
